clc; clear; close all;
% problem inputs
%% ====time stepping====
dt=1e-4;%CFL * min(lx,ly)/1;
t=0;Tmax =100.0;
Tmax=Tmax+dt;
yita=1;  %yita=0: Newtonian yita=1: non-Newtonian
%% ===========Flow parameters===============
Re=1e-2; beta=0.5;alph=0;gamma=0.9;Q0=20;We=0.01; 
xi=(1-beta)/(Re*We);
g=-9.8;Fr=0.1;
v0=-200;
%% ========space discretization==================
xa=0; xb=1; % x direction 
ya=0; yb=1; % y direction
za=0; zb=1; % z direction
nx=10; ny=10; nz=10; % elements at x-y-z directions
Ne=nx*ny*nz; % total elemennts
Ng=(nx+1)*(ny+1)*(nz+1); % total nodes
Lx=(xb-xa)/nx; % dx 
Ly=(yb-ya)/ny; % dy
Lz=(zb-za)/nz; % dz 
%% ======Ordering: (1) x y loop (2) z loop 
xe=(xa : Lx : xb)'; Ix=ones(nx+1,1); % xe: x position sampling
ye=(ya : Ly : yb)'; Iy=ones(ny+1,1); % ye: y position sampling
ze=(za : Lz : zb)'; Iz=ones(nz+1,1); % ze: z position sampling
xg=kron(Iz,kron(Iy,xe)); % xg: all nodes x position
yg=kron(Iz,kron(ye,Ix));% yg: all nodes x position
zg=kron(ze,kron(Iy,Ix));% yg: all nodes x position
%%==================
xmin=[xa,ya,za]; dx=[Lx,Ly,Lz]; nexyz=[nx,ny,ny];
Xg=[xg, yg, zg]; %all nodes position
Vg=zeros(3*Ng,1); % velocity on the grids- vector [Vx; Vy; Vz];
VBC=0*Vg; % BC value
%% connectivity matrix
icon=zeros(8, Ne);
%====first four nodes label on x-y layer======
for k=1:nz
    for j=1:ny
        for i=1:nx
            e=(k-1)*ny*nx + (j-1)*nx + i;
            icon(1,e) = (k-1)*(nx+1)*(ny+1) + (j-1)*(nx+1) + i;
            icon(2,e) = icon(1,e)+1;
            icon(3,e) = icon(2,e)+nx+1;
            icon(4,e) = icon(3,e)-1;
        end
    end
end
%=====adding one x-y face total nodes to get other four nodes
icon(5:8,:) = icon(1:4,:) + (nx+1)*(ny+1); %   
%% iniitalize the material point
nep=[8,8,8];
[Np, xp, vp]= InitialParticle(nep, dx, icon, Xg, Ne);  
Tp=zeros(Np,6);
pp=zeros(Np,1);
%===========================================
figure('Name', 'Initial mesh grid and mp');
hold on; pbaspect([1 1 1]);
for j=1:Ne
    id=icon(:,j);    id=[id; id(1)];
    plot3(Xg(id,1), Xg(id,2), Xg(id,3), 'r.', 'LineWidth', 1)
end

%% === boundary setup======================
ntop=find(Xg(:,3)>=1);
nbot=find(Xg(:,3)<=0);
nright=find(Xg(:,1)>=1);
nleft=find(Xg(:,1)<=0);
nfront=find(Xg(:,2)<=0);
nback=find(Xg(:,2)>=1);

NBCv=[nbot+2*Ng];
%% =============Assemble Matrix============
[M, K, Gx, Gy, Gz] = FEMmatrix3D(Ne, Ng, icon, Lx, Ly, Lz);    
zer=0*Gx;
G10= sparse([Gx,  zer,zer;
             zer, Gy, zer;
             zer, zer, Gz;
            0.5*Gy, 0.5* Gx, zer;
           0.5*Gz, zer, 0.5*Gx;
           zer, 0.5* Gz,  0.5*Gy]); %===Gradient of Velocity
G20= sparse([Gx; Gy; Gz]); %=====Gradient of P  
G30= sparse([Gx,  zer, zer, Gy, Gz, zer;
           zer, Gy,  zer, Gz, zer, Gx;
           zer, zer, Gz,  zer, Gx, Gy]); %==Divergence of stress
% full scale==
Mu0=[ M,zer, zer;
    zer, M, zer;
    zer, zer, M];% mass matrix
Mt0=[ M, zer, zer, zer, zer, zer;
    zer, M, zer, zer, zer, zer;
    zer, zer, M, zer, zer, zer;
    zer, zer, zer, M, zer, zer;
    zer, zer, zer, zer, M, zer;
    zer, zer, zer, zer, zer, M];
    
    
Mu=(sum(Mu0,2));
dMu=sparse(diag(1./Mu));
Mt0=diag(sum(Mt0,2));

%=================
 p=zeros(Ng,1); 
 vp(3,:)=v0; % initial particle velocity
 step=0;
 tic
while(step<=200)  
    step=step+1;
    %%% Map to grid%%%%%
    [xpn, nep]=natcoords(xp,dx,xmin,nexyz);
    [mv,vnew,Tnew,p,nn]=P2G(Ng,icon,xpn,nep,Np,vp,pp,Tp);
    for i=1:length(NBCv)
        if vnew(NBCv(i))<0        
        vnew(NBCv(i)) = VBC(NBCv(i));
        end
    end
    v=vnew;
    T=Tnew;
    %% ====
    [Hg,f]=Hstar(T, alph, gamma, Q0, xi, We, Ng, G10, vnew, Mt0);
    [Mf, A22d]=massA2(dx, Ng, Ne, icon, f, We, dt); 
  %% =====grids elimiation=====
      NBC=find(nn<=0);  % NBC: non particle nodes, NBS: free surface nodes  
      Phi=speye(Ng); Phi(:,NBC)=[];
      dxm=Phi'*Gx*Phi; dym=Phi'*Gy*Phi; dzm=Phi'*Gz*Phi;  
      nk=length(dxm);Ck=Phi'*M*Phi; zk=zeros(nk);
      [G1, D, G, Mu, Md, Mt, dMu, A22d]=full_scale_operators(dxm, dym, dzm, Ck, Mf, Phi, Re, beta, zk);     
      [Nk, vk, Hk, Tk, pk, znk]=Matrix_reduction(nn, nk, Ng, Phi, vnew, Hg, Tnew,p);

 %% ==========Free Surface================
   Gnx=dxm*Nk;   Gny=dym*Nk; Gnz=dzm*Nk;
   Gn=sqrt(Gnx.*Gnx+Gny.*Gny+Gnz.*Gnz);
   nnx=Gnx./(Gn); nny=Gny./(Gn); nnz=Gnz./(Gn);
   NBS=find(Gn>1e-8); nd=[nnx, nny, nnz];
  % X10=beta/Re*L*vk; X20=Tk; X30=pk;
   X10=beta/Re*G1*vk; X20=Tk; X30=pk;
   %% ===========Solve ============
   [A11, A12, A13, A21, A22, A23, A31, A32, A33, B1, B2, B3]= SolveAssemble(X20, Hk, dMu, Mt, A22d, NBS, beta, Re, G1, D, G, xi, nk, vk, dt);
   [X1, X2, X3]=SolveEuler(X10, X20, X30, B1, B2, B3, A11, A12, A13, A21, A22, A23, A31, A32, A33, beta, Re, NBS, nd, nk, yita);
   %% ==========Results Reterive====  
    zn=zeros(Ng,nk);
    Tg=[Phi, zn, zn, zn, zn, zn ;
        zn, Phi, zn, zn, zn, zn ;
        zn, zn, Phi, zn, zn, zn;
        zn, zn, zn, Phi, zn, zn ;
        zn, zn, zn, zn, Phi, zn ;
        zn, zn, zn, zn, zn, Phi ]*X2;
    p=Phi*X3;      
    dV=dMu*(beta/Re*D*X1+ D*X2- G*X3)*dt;  
    deltV=[Phi, zn, zn;
           zn, Phi, zn;
           zn, zn, Phi]*dV;
    vtilda=vnew+deltV;
    vnew=vtilda;
   %% ========= Boundary Condition==========
    for i=1:length(NBCv)
        if vnew(NBCv(i))<0        
        vnew(NBCv(i)) = VBC(NBCv(i));
        end
    end
    %% ===== Grids to Particles=============
    [xp,vp,Tp,pp]=maptopointsPC(Ng,Tg,xmin,nexyz,Np,xp,vp,icon,vnew,v,dx,dt, p,Fr,g);
    v=vnew;  
    %% ====Plot======
    clf  
    plot3(xp(1,:), xp(2,:), xp(3,:), 'b.', 'LineWidth', 2) 
    axis([0.2 0.8 0.2 0.8 0 0.3])
    xlabel('X'); ylabel('Y'); zlabel('Z')
    view(-53, 49)
    drawnow 
end  
  toc  
