%% 3D mesh
clc; clearvars; close all;
xa=0; xb=1; % x direction 
ya=0; yb=1; % y direction
za=0; zb=1; % z direction
nx=2; ny=2; nz=2; % elements at x-y-z directions
Ne=nx*ny*nz; % total elemennts
Ng=(nx+1)*(ny+1)*(nz+1); % total nodes
Lx=(xb-xa)/nx; % dx 
Ly=(yb-ya)/ny; % dy
Lz=(zb-za)/nz; % dz 
%%=====coordinates of nodes=======
%%======Ordering: (1) x y loop (2) z loop 
xe=(xa : Lx : xb)'; Ix=ones(nx+1,1); % xe: x position sampling
ye=(ya : Ly : yb)'; Iy=ones(ny+1,1); % ye: y position sampling
ze=(za : Lz : zb)'; Iz=ones(nz+1,1); % ze: z position sampling
xg=kron(Iz,kron(Iy,xe)); % xg: all nodes x position
yg=kron(Iz,kron(ye,Ix));% yg: all nodes x position
zg=kron(ze,kron(Iy,Ix));% yg: all nodes x position
%% connectivity matrix
con=zeros(8, Ne);
%====first four nodes label on x-y layer======
for k=1:nz
    for j=1:ny
        for i=1:nx
            e=(k-1)*ny*nx + (j-1)*nx + i;
            con(1,e) = (k-1)*(nx+1)*(ny+1) + (j-1)*(nx+1) + i;
            con(2,e) = con(1,e)+1;
            con(3,e) = con(2,e)+nx+1;
            con(4,e) = con(3,e)-1;
        end
    end
end
%=====adding one x-y face total nodes to get other four nodes
con(5:8,:) = con(1:4,:) + (nx+1)*(ny+1); %
%% FEM matrix
[M, K, Gx, Gy, Gz] = FEMmatrix3D(Ne, Ng, con, Lx, Ly, Lz);
%% boundary
tol=1E-8;
NBC=[]; Tbc=zeros(Ng,1);
for j=1:Ng
    if abs(xg(j)-xa)<tol %% finding bc nodes
        NBC=[NBC; j];
    elseif abs(xg(j)-xb)<tol
        NBC=[NBC; j];
    elseif abs(yg(j)-ya)<tol
        NBC=[NBC; j];
    elseif abs(yg(j)-yb)<tol
        NBC=[NBC; j];
    elseif abs(zg(j)-za)<tol
        NBC=[NBC; j];
    elseif abs(zg(j)-zb)<tol
        NBC=[NBC; j];
    else
    end
end
pod=speye(Ng); pod(:,NBC)=[];% pod: phi in NN fluid

%% =======Nature coordinates for particles in element
xp=[0.6;0.7;0.2];
xmin=[0,0,0]; dx=[Lx,Ly,Lz]; nexyz=[nx,ny,ny];
Xg=[xg, yg, zg]; %all nodes position
[xpn, nep]=natcoords(xp,dx,xmin,nexyz, Xg, con);

%% ==============================================

%% shape function
function N=shape3D(x, y, z)
    N1=(1-x)*(1-y)*(1-z)/8;
    N2=(1+x)*(1-y)*(1-z)/8;
    N3=(1+x)*(1+y)*(1-z)/8;
    N4=(1-x)*(1+y)*(1-z)/8;
    N5=(1-x)*(1-y)*(1+z)/8;
    N6=(1+x)*(1-y)*(1+z)/8;
    N7=(1+x)*(1+y)*(1+z)/8;
    N8=(1-x)*(1+y)*(1+z)/8;
    N=[N1, N2, N3, N4, N5, N6, N7, N8];
end


function [xpn, nep]=natcoords(xp,dx,xmin,ne, Xg, con)
np=size(xp,2);
for ip=1:np
    xx=(xp(:,ip)-xmin')./dx';
    i=fix(xx); % x y z element rough label   
    nep(:,ip)=i(1)+i(2)*ne(1)+i(3)*(ne(2)*ne(1))+1; % element label
    id=nep(:,ip);
    xg=i'.*dx;%Xg(con(1,id),:); % left-front-bottom node
    xpn(:,ip)=2*(xp(:,ip)-xg')./dx'-1; % relative position to element center node
end

end

